#ifndef _CHAMPLAIN_PERL_H_

#include <clutterperl.h>


#include <champlain/champlain.h>

#ifdef CHAMPLAINPERL_GTK
#include <gtk2perl.h>
#include <champlain-gtk/champlain-gtk.h>
#endif


#include "champlain-autogen.h"

#endif /* _CHAMPLAIN_PERL_H_ */
